/************************************************************************

 The contents of this file are subject to the Mozilla Public License
 Version 1.1 (the "License"); you may not use this file except in	
 compliance with the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS"
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 License for the specific language governing rights and limitations
 under the License.

 The Original Code is Isearch.

 The Initial Developer of the Original Code is Clearinghouse for
 Networked Information Discovery and Retrieval.

 Portions created by Netscape Communications are Copyright (C) 2001. 
 All Rights Reserved.

 Portions created by the Clearinghouse for Networked Information
 Discovery and Retrieval are Copyright (C) 1994. All Rights Reserved.

************************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include "keyword.h"

GPTYPE 
KEYWORD::ParseWords(unsigned char* DataBuffer, // Pointer to document text buffer.
		    int DataLength,            // Length of document text buffer in # of characters.
		    int DataOffset,            // Offset that must be added to all GP positions 
		                               // because GP space is shared with other documents. 
		    unsigned int FileId,       // file identification number
		    int *fdList)               // list of open files to write data to

{
    index_entry ie;
    int fn;
    int numAdded = 0;
    int Position = 0;
    int linenum = 0;

    // get past the tag 
    while ((Position < DataLength) && (DataBuffer[Position] != ':'))
	Position++;
    
    if (Position < DataLength)
	Position++;
    
    while (Position < DataLength) {
	// scan to the begining of the word
	while ((Position < DataLength) &&
	       (!IsAlnum(DataBuffer[Position])) &&
	       (!IsUTF8_Start_Char(DataBuffer[Position]))) {
	    
	    if (DataBuffer[Position] == '\n')
		linenum++;
	    Position++;
	}
	
	// remember this offset as the beginning of a word
	if ((linenum < 1) && (Position < DataLength)) {

	    fn = GetBucket(DataBuffer[Position]);

	    ie.gpindex = DataOffset + Position;
	    ie.mdtindex = FileId;
	    
	    if ( sizeof(index_entry) != write(fdList[fn], (void *)&ie, sizeof(index_entry))) {
		fprintf(stderr,"ERROR: can't write to fd %d\n");
		exit(88);
	    }
	    numAdded++;
	    
	}
	
	// scan to the end of the word, delimited by white space
	while ((Position < DataLength) &&
	       (!isspace(DataBuffer[Position])))  {
	    Position++;
	}
    }
    return numAdded;
} 

double 
KEYWORD::ScoreHit(IRESULT result, double termweight) {
  
    double Weight = 1.0;

    if (result.MiscFields & COOL_FLAG_MASK) {
        Weight = 3.0;
    }

    if (result.HitCount < 1)
        result.HitCount = 1;

    // adjust score for the number of hits in this record, range [1.0, 2.0)
    Weight *= ((double)2.0 - ((double)1.0 / ((double)result.HitCount)));

    return(termweight * Weight);
}  

unsigned int 
KEYWORD::TopLevelCat(unsigned char *q) {
  
  /* by default use the first three characters of the category as
   * a hash
   */
  if (q)
    return((q[0] << 16) + (q[1] << 8) + q[2]);
  else
    return(0);
}

KEYWORD::KEYWORD (PIDBOBJ DbParent): DOCTYPE (DbParent){}

KEYWORD::~KEYWORD () {}

