/************************************************************************

 The contents of this file are subject to the Mozilla Public License
 Version 1.1 (the "License"); you may not use this file except in	
 compliance with the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS"
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 License for the specific language governing rights and limitations
 under the License.

 The Original Code is Isearch.

 The Initial Developer of the Original Code is Clearinghouse for
 Networked Information Discovery and Retrieval.

 Portions created by Netscape Communications are Copyright (C) 2001. 
 All Rights Reserved.

 Portions created by the Clearinghouse for Networked Information
 Discovery and Retrieval are Copyright (C) 1994. All Rights Reserved.

************************************************************************/

/*
 * Data structure to store result data. 
 */

#include "defs.h"
#include "string.h"
#include "vlist.h"
#include "mdtrec.h"
#include "mdt.h"
#include "iresult.h"
#include "mdthashtable.h"
#include "catids.h"

extern MDTHASHTABLE ht;

IRESULT::IRESULT() {
	MdtIndex = 0;
	HitCount = 0;
	RealHitCount = 1;
	Score = 0.0;
        pathName = 0;
        fileName = 0;
        GlobalFileStart = 0;
        GlobalFileEnd   = 0;

	TopLevelCat  = TOP_CAT_None;
	DocumentType = 0;
	MiscFields   = 0;
	CatDepth     = 1;
	Field        = 0;

}

IRESULT& IRESULT::operator=(const IRESULT& OtherIresult) {
	MdtIndex = OtherIresult.MdtIndex;
	HitCount = OtherIresult.HitCount;
	RealHitCount = OtherIresult.RealHitCount;

        pathName = OtherIresult.pathName;
        fileName = OtherIresult.fileName;
        GlobalFileStart = OtherIresult.GlobalFileStart;
        GlobalFileEnd   = OtherIresult.GlobalFileEnd;

	TopLevelCat  = OtherIresult.TopLevelCat;
	DocumentType = OtherIresult.DocumentType;
	MiscFields   = OtherIresult.MiscFields;
	CatDepth     = OtherIresult.CatDepth;
	Field        = OtherIresult.Field;

	Score = OtherIresult.Score;
	return *this;
}

void IRESULT::SetMdtIndex(const INT NewMdtIndex) {
	MdtIndex = NewMdtIndex;
}

INT IRESULT::GetMdtIndex() const {
	return MdtIndex;
}

void IRESULT::SetHitCount(const INT NewHitCount) {
	HitCount = NewHitCount;
}

void IRESULT::IncHitCount() {
	HitCount++;
}

void IRESULT::IncHitCount(const INT AddCount) {
	HitCount += AddCount;
}

void IRESULT::IncRealHitCount(const INT RealCount) {
	RealHitCount += RealCount;
}

INT IRESULT::GetHitCount() const {
	return HitCount;
}

void IRESULT::SetScore(const DOUBLE NewScore) {
	Score = NewScore;
}

void IRESULT::IncScore(const DOUBLE AddScore) {
	Score += AddScore;
}

void IRESULT::AndIncScore(const DOUBLE AddScore) {
	Score += AddScore;
	Score *= 1.5;
}

void IRESULT::OrIncScore(const DOUBLE AddScore) {
        // Take the Maximum of both scores
        if (Score < AddScore)
	  Score = AddScore;
}

DOUBLE IRESULT::GetScore() const {
	return Score;
}

void IRESULT::SetDocumentType(unsigned char NewDocumentType) {
	DocumentType = NewDocumentType;
}

unsigned char IRESULT::GetDocumentType() const {
	return DocumentType;
}

void IRESULT::GetPathName(PSTRING StringBuffer) const {
	*StringBuffer = ht.GetPathName(pathName);
}

// void IRESULT::SetFileName(const STRING& NewFileName) {
//	fileName = ht.AddFileName(NewFileName);
//}

void IRESULT::GetFileName(PSTRING StringBuffer) const {
	*StringBuffer = ht.GetFileName(fileName);
}

void IRESULT::GetFullFileName(PSTRING StringBuffer) const {
        char *tmp;
	tmp = ht.GetPathName(pathName);
	*StringBuffer = tmp;
	// XXX - free(tmp);

	tmp = ht.GetFileName(fileName);
	StringBuffer->Cat(tmp);
	// XXX - free(tmp);	
}

unsigned char IRESULT::IsCool() const {
  return (MiscFields & (unsigned char) COOL_FLAG_MASK);
}

unsigned char IRESULT::IsDeleted() const {
  return (MiscFields & (unsigned char) DELETED_FLAG_MASK);
}

unsigned char IRESULT::IsCommerce() const {
  return (MiscFields & (unsigned char) COMMERCE_FLAG_MASK);
}

unsigned char  IRESULT::GetMiscField() const {
  return (MiscFields & FIELD_FLAG_MASK);
}

unsigned char IRESULT::GetField() const {
  return (Field);
}

unsigned char IRESULT::GetCatDepth() const {
  if (CatDepth)
    return CatDepth;
  else
    return 1;
}


IRESULT::~IRESULT() { }
