/************************************************************************

 The contents of this file are subject to the Mozilla Public License
 Version 1.1 (the "License"); you may not use this file except in	
 compliance with the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS"
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 License for the specific language governing rights and limitations
 under the License.

 The Original Code is Isearch.

 The Initial Developer of the Original Code is Clearinghouse for
 Networked Information Discovery and Retrieval.

 Portions created by Netscape Communications are Copyright (C) 2001. 
 All Rights Reserved.

 Portions created by the Clearinghouse for Networked Information
 Discovery and Retrieval are Copyright (C) 1994. All Rights Reserved.

************************************************************************/

/* Description:	Internal Search Result Set */

#ifndef IRSET_H
#define IRSET_H

#include "idbobj.h"

typedef struct I_LIST {
  int           irset_index;    /* irset table index */
  int           mdt_index;      /* mdt index value */
  struct I_LIST *next;          /* next list item */
} i_list;

class IRSET {
public:
	IRSET(const PIDBOBJ DbParent);
	IRSET(const PIDBOBJ DbParent,int start_size);
	void Init(const PIDBOBJ DbParent,int start_size);
	IRSET& operator=(const IRSET& OtherIrset);
	INT GetOperandType() const { return TypeRset; };
	IRSET* Duplicate() const;
	IRSET* Duplicate();
	void AddEntry(const IRESULT& ResultRecord, const INT AddHitCounts);
	void FastAddEntry(const IRESULT& ResultRecord, const INT AddHitCounts);
	void GetEntry(const INT Index, PIRESULT ResultRecord) const;
	void Expand();
	void CleanUp();
	void Resize(const INT Entries);
	INT GetTotalEntries() const;
	INT GetHitTotal() const;
	void ComputeScores(const double TermWeight, const int total_count );

	void Or(IRSET *OtherIrset);
	void And(IRSET *OtherIrset);
	void AndNot(IRSET *OtherIrset);

        // Near is hardcoded to be 50 characters
        void Near(const IRSET & OtherIrset) { CharProx(OtherIrset, 50); }
        void CharProx(const IRSET& OtherIrset, const INT Distance);
	void SortByScore();
	void SortByIndex();
	void SetParent(PIDBOBJ const NewParent);
	double GetMaxScore();
	double GetMinScore();
	PIDBOBJ GetParent() const;
        void SetRestrictDir(const class STRING &);
        void SetTopLevelRestrictDir(const unsigned int);
        void SetRestrictAdult(int i);
	~IRSET();
        void HashInit();
        int Hash(int  mdt_index);
        int HashAdd(int mdt_index);
        int HashGet(int mdt_index);
        int GetScaledScore(const double UnscaledScore, const INT ScaleFactor);

	// retrict searches to a sub directory
	STRING RestrictDir;
	// the id of the top level directory 
	unsigned int TopLevelRestrictDir;
	// retrict Adult searches
	int RestrictAdult;
	// Maximum length of record 
	int          MaxRecordLength;
	
	int     tableSize;
	struct I_LIST **ht;

	PIDBOBJ  Parent;
	PIRESULT Table;
	int      TotalEntries;
	int      MaxEntries;
	int      ScoreSort;
	double   MaxScore,MinScore;

#ifdef DEBUG_HASH_COLLISIONS
  int collisions;
#endif
};

typedef IRSET* PIRSET;

#endif
