/************************************************************************

 The contents of this file are subject to the Mozilla Public License
 Version 1.1 (the "License"); you may not use this file except in	
 compliance with the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS"
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 License for the specific language governing rights and limitations
 under the License.

 The Original Code is Isearch.

 The Initial Developer of the Original Code is Clearinghouse for
 Networked Information Discovery and Retrieval.

 Portions created by Netscape Communications are Copyright (C) 2001. 
 All Rights Reserved.

 Portions created by the Clearinghouse for Networked Information
 Discovery and Retrieval are Copyright (C) 1994. All Rights Reserved.

************************************************************************/

/* Description:	Class MDTREC - Multiple Document Table Record */

#include <sys/../string.h>

#include "mdtrec.h"
#include "catids.h"
#include "mdthashtable.h"

/*---------------------------------------------------------
 * The MDT hashtable is shared globally by all instances
 * of MDTREC. 
 *---------------------------------------------------------
 */

extern MDTHASHTABLE ht;

MDTREC::MDTREC() {
  DocumentType = 0;
  MiscFields = 0;
  CatDepth = 1;
  Language = LANGUAGE_English;
  TopLevelCat  = TOP_CAT_None;
  pathName = 0;
  fileName = 0;

  GlobalFileStart = 0;
  GlobalFileEnd = 0;
  LocalRecordStart = 0;
  LocalRecordEnd = 0;

}

 
MDTREC& 
MDTREC::operator=(const MDTREC& OtherMdtRec) {
  DocumentType = OtherMdtRec.DocumentType;
  MiscFields   = OtherMdtRec.MiscFields;
  CatDepth     = OtherMdtRec.CatDepth;
  Language     = OtherMdtRec.Language; 
  TopLevelCat  = OtherMdtRec.TopLevelCat;

  pathName = OtherMdtRec.pathName;
  fileName = OtherMdtRec.fileName;

  GlobalFileStart  = OtherMdtRec.GlobalFileStart;
  GlobalFileEnd    = OtherMdtRec.GlobalFileEnd;
  LocalRecordStart = OtherMdtRec.LocalRecordStart;
  LocalRecordEnd   = OtherMdtRec.LocalRecordEnd;
  return *this;
}


void 
MDTREC::FlipBytes() {

  // XXX need fix byte order one day
  //GpSwab(&fileName);
  //GpSwab(&pathName);

  GpSwab(&GlobalFileStart);
  GpSwab(&GlobalFileEnd);
  GpSwab(&LocalRecordStart);
  GpSwab(&LocalRecordEnd);
}

void 
MDTREC::SetDocumentType(const unsigned char NewDocumentType) {
  DocumentType =  NewDocumentType;
}


unsigned char
MDTREC::GetDocumentType() const {
  return DocumentType;
}


void 
MDTREC::SetPathName(const STRING& NewPathName) {

  // cerr << "SETTING PATHNAME = [" <<  PathName << FileName << "]\n";
  pathName = ht.AddPathName(NewPathName);

}


void 
MDTREC::GetPathName(STRING* StringBuffer) const {
  *StringBuffer = ht.GetPathName(pathName);
}


void 
MDTREC::SetFileName(const STRING& NewFileName) {

  // cerr << "SETTING FILENAME = [" <<  PathName << FileName << "]\n";
  fileName = ht.AddFileName(NewFileName);

}


void 
MDTREC::GetFileName(STRING* StringBuffer) const {
  *StringBuffer = ht.GetFileName(fileName);;
}


void 
MDTREC::GetFullFileName(STRING* StringBuffer) const {
  GetPathName(StringBuffer);
  StringBuffer->Cat(ht.GetFileName(fileName));
}


void 
MDTREC::SetGlobalFileStart(const GPTYPE NewGlobalFileStart) {
  GlobalFileStart = NewGlobalFileStart;
}


GPTYPE 
MDTREC::GetGlobalFileStart() const {
  return GlobalFileStart;
}


void 
MDTREC::SetGlobalFileEnd(const GPTYPE NewGlobalFileEnd) {
  GlobalFileEnd = NewGlobalFileEnd;
}


GPTYPE 
MDTREC::GetGlobalFileEnd() const {
  return GlobalFileEnd;
}


void 
MDTREC::SetLocalRecordStart(const GPTYPE NewLocalRecordStart) {
  LocalRecordStart = NewLocalRecordStart;
}


GPTYPE 
MDTREC::GetLocalRecordStart() const{
  return LocalRecordStart;
}


void 
MDTREC::SetLocalRecordEnd(const GPTYPE NewLocalRecordEnd) {
  LocalRecordEnd = NewLocalRecordEnd;
}


GPTYPE 
MDTREC::GetLocalRecordEnd() const {
  return LocalRecordEnd;
}


void 
MDTREC::SetDeleted(const GDT_BOOLEAN Flag) {
  if (Flag) {
    MiscFields |= DELETED_FLAG_MASK ;
  } else {
    MiscFields &= ~DELETED_FLAG_MASK;
  }
}


GDT_BOOLEAN 
MDTREC::GetDeleted() const {
  return (MiscFields & DELETED_FLAG_MASK) ? GDT_TRUE : GDT_FALSE;
}


MDTREC::~MDTREC() {
}
